<?php

/**
 * Plugin Name: Ecomdy Pixel
 * Description: Connects Pixel Tiktok to WooCommerce.
 * Version: 1.0.0
 * Author: Ecomdy Media
 * License: GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 */
if (!defined('ABSPATH')) exit;
register_activation_hook(__FILE__, 'ctecd_tiktok_pixel_install');

function ctecd_tiktok_pixel_install()
{
  global $wpdb;
  $table_name = $wpdb->prefix . 'tiktok_pixel';

  $charset_collate = $wpdb->get_charset_collate();

  $sql = "CREATE TABLE $table_name (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        pixel_name VARCHAR(255) NOT NULL,
        pixel_id VARCHAR(255) NOT NULL,
        real_id  VARCHAR(255) NOT NULL,
        pixel_events VARCHAR(255) NOT NULL,
        pixel_status BOOLEAN NOT NULL,
        updated DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
        PRIMARY KEY (id)
    ) $charset_collate;";

  require_once ABSPATH . 'wp-admin/includes/upgrade.php';
  dbDelta($sql);
}
add_action('wp_head', 'ctecd_add_tiktok_pixel_script');

function ctecd_add_tiktok_pixel_script()
{
  $pixels = ctecd_get_tiktok_pixel();
  if ($pixels) {
    foreach ($pixels as $pixel) {
      $pixel_id = esc_js($pixel['pixel_id']);
      $real_id = esc_js($pixel['real_id']);
      $pixel_status = esc_js($pixel['pixel_status']);
      if (strlen($pixel_id) == 24) {
        if (ctecd_check_timeout($pixel['id'])) {
          $result = ctecd_get_official_pixel_async($pixel_id)['result'];
          if ($result["status"] == "OK" && $result["pixelStatus"]=="RUNNING") {
            $pixel_id = $result["pixelCode"];
            ctecd_update_tiktok_pixel_id($pixel['id'], $pixel_id);
            ctecd_update_tiktok_pixel_timeout($pixel['id']);
          } else {
            ctecd_update_tiktok_pixel_id($pixel['id'], $pixel_id,false);
            ctecd_update_tiktok_pixel_timeout($pixel['id']);
          }
        }
      }
      if (strlen($real_id) != 24) {
?>
<script>
! function(w, d, t) {
    w.TiktokAnalyticsObject = t;
    var ttq = w[t] = w[t] || [];
    ttq.methods = ["track", "identify", "instances", "debug", "on", "off", "once", "ready", "alias", "group",
        "enableCookie", "disableCookie"
    ], ttq.setAndDefer = function(t, e) {
        t[e] = function() {
            t.push([e].concat(Array.prototype.slice.call(arguments, 0)))
        }
    };
    for (var i = 0; i < ttq.methods.length; i++) ttq.setAndDefer(ttq, ttq.methods[i]);
    ttq.instance = function(t) {
        for (var e = ttq._i[t] || [], n = 0; n < ttq.methods.length; n++) ttq.setAndDefer(e, ttq.methods[n]);
        return e
    }, ttq.load = function(e, n) {
        var i = "https://analytics.tiktok.com/i18n/pixel/events.js";
        ttq._i = ttq._i || {}, ttq._i[e] = [], ttq._i[e]._u = i, ttq._t = ttq._t || {}, ttq._t[e] = +new Date, ttq
            ._o = ttq._o || {}, ttq._o[e] = n || {};
        n = document.createElement("script");
        n.type = "text/javascript", n.async = !0, n.src = i + "?sdkid=" + e + "&lib=" + t;
        e = document.getElementsByTagName("script")[0];
        e.parentNode.insertBefore(n, e)
    };
    ttq.load('<?php echo esc_js($real_id); ?>');
}(window, document, 'ttq');
</script>
<?php
      }
    }
  }
}

function ctecd_get_official_pixel_async($pixel_id)
{
  // Retrieve official TikTok Pixel ID from Ecomdy Media's ecosystem.
  // This API call fetches the official TikTok Pixel ID associated with the provided Pixel ID from Ecomdy Media's ecosystem.
  // It is used to verify and ensure that the provided Pixel ID is valid and corresponds to the expected TikTok Pixel ID.
  $url = "https://ads.ecomdymedia.com/api/eac/pixel-public/tiktok-code?pixelId=" . $pixel_id;
  $response = wp_remote_get($url);
  if (is_wp_error($response)) {
    return ['error' => 'Failed to fetch data from the API.'];
  }
  $body = wp_remote_retrieve_body($response);
  $decoded_body = json_decode($body, true);
  return $decoded_body;
}

function ctecd_update_tiktok_pixel_id($id, $new_pixel_id, $new_pixel_status = true)
{
  global $wpdb;
  $table_name = $wpdb->prefix . 'tiktok_pixel';

  $wpdb->update(
    $table_name,
    array(
      'real_id'    => $new_pixel_id,
      'pixel_status' => $new_pixel_status,
    ),
    array('id' => $id),
    array('%s', '%d'), // Format for pixel_id and pixel_status fields
    array('%d')
  );
}
function ctecd_update_tiktok_pixel_timeout($id)
{
  global $wpdb;
  $table_name = $wpdb->prefix . 'tiktok_pixel';

  $wpdb->update(
    $table_name,
    array(
      'updated'  => current_time('mysql', 1), // Set the updated field to the current timestamp
    ),
    array('id' => $id),
    array('%s'), // Format for the updated field
    array('%d')
  );
}

function ctecd_check_timeout($id)
{
  global $wpdb;
  $table_name = $wpdb->prefix . 'tiktok_pixel';

  // Get the current time in WordPress format
  $current_time = current_time('mysql', 1);

  // Calculate the time 10 minutes ago
  $ten_minutes_ago = date('Y-m-d H:i:s', strtotime('-10 minutes', strtotime($current_time)));

  // Properly escape the table name to prevent SQL injection (though specific for table names, manual input is generally safe)
  $table_name_escaped = esc_sql($table_name);

  // Check if the last update was more than 10 minutes ago
  // Directly using the table name in the query since placeholders cannot be used for table names
  $query = "SELECT updated FROM `{$table_name_escaped}` WHERE id = %d";
  $last_updated = $wpdb->get_var($wpdb->prepare($query, $id));

  return strtotime($last_updated) < strtotime($ten_minutes_ago);
}

function ctecd_add_tiktok_viewcontent_event()
{
  if (is_product()) {
    global $product;
    $product_id = $product->get_id();
    $product_price = $product->get_price();
    $currency = get_woocommerce_currency();
    ?>
<script>
document.addEventListener('DOMContentLoaded', function() {
    ttq.track('ViewContent', {
        content_id: '<?php echo esc_js($product_id); ?>',
        content_type: 'product',
        value: '<?php echo esc_js($product_price); ?>',
        currency: '<?php echo esc_js($currency); ?>'
    });
});
</script>
<?php
  }
}
add_action('wp_footer', 'ctecd_add_tiktok_viewcontent_event', 20);

add_action('woocommerce_add_to_cart', 'ctecd_wp_kama_woocommerce_add_to_cart_action', 10, 6);

function ctecd_wp_kama_woocommerce_add_to_cart_action($cart_id, $product_id, $request_quantity, $variation_id, $variation, $cart_item_data)
{
  $product = wc_get_product($product_id);
  if (!$product) {
    return;
  }
  ?>
    <script>
      document.addEventListener('DOMContentLoaded', function() {
          var event_data = {
              'price': <?php echo esc_js((float) $product->get_price()); ?>,
              'currency': '<?php echo esc_js(get_woocommerce_currency()); ?>',
              'content_name': '<?php echo esc_js($product->get_name()); ?>',
              'quantity': <?php echo esc_js((int) $request_quantity); ?>,
              'content_type': 'product',
              'content_id': '<?php echo esc_js($product_id); ?>',
              'value': <?php echo esc_js((float) $product->get_price() * (float) $request_quantity); ?>,
          };
          if (typeof ttq !== 'undefined' && event_data.price && event_data.quantity) {
              ttq.track('AddToCart', event_data);
          }
      });
    </script>
  <?php
}

function ctecd_wp_kama_woocommerce_loaded_action()
{
  add_filter('woocommerce_cart_needs_shipping', 'ctecd_wp_kama_woocommerce_cart_needs_shipping_filter');
}

add_action('woocommerce_loaded', 'ctecd_wp_kama_woocommerce_loaded_action');
function ctecd_wp_kama_woocommerce_cart_needs_shipping_filter($needs_shipping)
{
  if (!is_checkout() || did_action('wp_footer')) {
    remove_filter('woocommerce_cart_needs_shipping', 'ctecd_wp_kama_woocommerce_cart_needs_shipping_filter');
    return;
  }

  // Removed Script on Every Ajax Request on Checkout Page By M Usman Rashid
  if(wp_doing_ajax() ){
    remove_filter('woocommerce_cart_needs_shipping', 'ctecd_wp_kama_woocommerce_cart_needs_shipping_filter');
    return; 
  }

  remove_filter('woocommerce_cart_needs_shipping', 'ctecd_wp_kama_woocommerce_cart_needs_shipping_filter');

  $cart_contents = WC()->cart->get_cart();
  $js_event_data = [];
  $value = 0;
  foreach ($cart_contents as $cart_item) {
    $product = $cart_item['data'];
    $quantity = (int)$cart_item['quantity'];
    $price = $product->get_price();
    $content_id = (string)$product->get_sku();

    if ($content_id === '') {
      $content_id = (string)$product->get_id();
    }

    // Add product data to the JavaScript object
    $js_event_data[] = [
      'price' => $price,
      'content_name' => $product->get_name(),
      'content_id' => $content_id,
      'content_type' => 'product',
      'quantity' => $quantity,
    ];

    // Calculate total value
    $value += $price * $quantity;
  }

?>
<script>
document.addEventListener('DOMContentLoaded', function() {
    var event_data = {
        'contents': <?php echo wp_json_encode($js_event_data); ?>,
        'currency': '<?php echo esc_js(get_woocommerce_currency()); ?>',
        'value': <?php echo esc_js($value); ?>,
    };

    // Check if ttq is defined before tracking
    if (typeof ttq !== 'undefined' && event_data.contents.length > 0) {
        ttq.track('InitiateCheckout', event_data);
    }
});
</script>
<?php
  return;
}

add_action('woocommerce_thankyou', 'ctecd_function_after_payment_complete', 10, 1);

function ctecd_function_after_payment_complete($order_id)
{
  $order = wc_get_order($order_id);
  if (!$order) {
    return;
  }
  $js_event_data = [];
  $value = 0;

  foreach ($order->get_items() as $item) {
    $product = $item->get_product();
    $quantity = (int)$item->get_quantity();
    $price = $product->get_price();
    $content_id = (string)$product->get_sku();

    if ($content_id === '') {
      $content_id = (string)$product->get_id();
    }
    $js_event_data[] = [
      'price' => $price,
      'content_name' => $product->get_name(),
      'content_id' => $content_id,
      'content_type' => 'product',
      'quantity' => $quantity,
    ];
    $value += $price * $quantity;
  }

?>
<script>
document.addEventListener('DOMContentLoaded', function() {
    var event_data = {
        'contents': <?php echo wp_json_encode($js_event_data); ?>,
        'currency': '<?php echo esc_js(get_woocommerce_currency()); ?>',
        'value': <?php echo esc_js($value); ?>,
    };
    if (typeof ttq !== 'undefined' && event_data.contents.length > 0) {
        ttq.track('Purchase', event_data);
    }
});
</script>
<?php
}

function ctecd_get_pixel_by_id($pixel_id)
{
  global $wpdb;
  $table_name = $wpdb->prefix . 'tiktok_pixel';

  // Prepare and execute a SELECT query to get pixel data by ID
  $query = $wpdb->prepare("SELECT * FROM {$wpdb->prefix}{$table_name} WHERE id = %d", $pixel_id);
  $pixel_data = $wpdb->get_row($query, ARRAY_A);

  return $pixel_data;
}

add_action('init', 'ctecd_process_pixel_action_post');

function ctecd_process_pixel_action_post()
{
  if (isset($_POST['submit_pixel']) && isset($_POST['save_pixel_nonce_field']) && wp_verify_nonce(sanitize_key($_POST['save_pixel_nonce_field']), 'save_pixel_nonce')) {
    // Retrieve form data
    $pixel_name = sanitize_text_field($_POST['pixel_name']);
    $pixel_id = sanitize_text_field($_POST['pixel_id']);
    $real_id = sanitize_text_field($_POST['pixel_id']);
    $event = sanitize_text_field($_POST['event']);
    $status = (strlen($pixel_id) == 24) ? 0 : 1;

    if (strlen($pixel_id) == 24) {
      $result = ctecd_get_official_pixel_async($pixel_id)['result'];
      if ($result["status"] == "OK" && $result["pixelStatus"]=="RUNNING") {
        $real_id = $result["pixelCode"];
        $status = 1;
      }
    }
    // Validate and save data to the database or perform any other necessary actions
    if ($pixel_name && $pixel_id && $event) {
      ctecd_save_pixel($pixel_name, $pixel_id,$real_id, $event, $status);
    }
  }
}
add_action('init', 'ctecd_process_pixel_action');

function ctecd_process_pixel_action()
{
  // Check if action and id parameters are present in the GET request
  if (isset($_GET['action']) && isset($_GET['id'])) {
    $action = sanitize_text_field($_GET['action']);
    $pixel_id = absint($_GET['id']);

    if ($action === 'edit') {
      // Perform action when edit is requested (if needed)
      // Add your code here for edit action
      exit;
    } elseif ($action === 'delete') {
      // Handle delete action, delete the pixel from the database or perform actions
      ctecd_delete_pixel($pixel_id);
      // Redirect to a specific admin page after deleting the pixel
      wp_redirect(admin_url('admin.php?page=tiktok-pixel'));
      exit;
    }
  }
}

// Function to delete a pixel (example)
function ctecd_delete_pixel($pixel_id)
{
  global $wpdb;
  $table_name = $wpdb->prefix . 'tiktok_pixel';

  $wpdb->delete(
    $table_name,
    array('id' => $pixel_id),
    array('%d')
  );
}
function ctecd_save_pixel($pixel_name, $pixel_id, $real_id, $event, $status)
{
  global $wpdb;
  $table_name = $wpdb->prefix . 'tiktok_pixel';

  $wpdb->insert(
    $table_name,
    array(
      'pixel_name' => $pixel_name,
      'pixel_id' => $pixel_id,
      'real_id' => $real_id,
      'pixel_events' => $event,  
      'pixel_status' => $status,
    ),
    array('%s', '%s', '%s', '%s', '%d') // Adjusted data types
  );
}
function ctecd_get_tiktok_pixel()
{
  global $wpdb;
  $table_name = $wpdb->prefix . 'tiktok_pixel';

  // No need for prepare as there are no dynamic values
  $query = "SELECT * FROM {$table_name}";

  // Execute the query
  $results = $wpdb->get_results($query, ARRAY_A);

  // Return the results
  return $results;
}

function ctecd_manage_scripts_shortcode()
{
  ob_start();
  include(plugin_dir_path(__FILE__) . 'views/list-pixel.php');
  return ob_get_clean();
}
add_shortcode('ctecd_manage_scripts', 'ctecd_manage_scripts_shortcode');


add_action('admin_menu', 'ctecd_script_manager_menu');

function ctecd_script_manager_menu()
{
  add_menu_page(
    'Ecomdy Pixel',
    'Ecomdy Pixel',
    'manage_options',
    'tiktok-pixel',
    'tiktok_pixel_page',
    plugin_dir_url(__FILE__) . 'assets/images/tiktok-logo.svg',
  );
}

function tiktok_pixel_page()
{
  include(plugin_dir_path(__FILE__) . 'views/tiktok-pixel.php');
}